# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from ... import _utilities
from . import outputs
from ... import meta as _meta
from ._inputs import *

__all__ = ['DeviceClassListArgs', 'DeviceClassList']

@pulumi.input_type
class DeviceClassListArgs:
    def __init__(__self__, *,
                 items: pulumi.Input[Sequence[pulumi.Input['DeviceClassArgs']]],
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input['_meta.v1.ListMetaArgs']] = None):
        """
        The set of arguments for constructing a DeviceClassList resource.

        :param pulumi.Input[Sequence[pulumi.Input['DeviceClassArgs']]] items: Items is the list of resource classes.
        :param pulumi.Input[_builtins.str] api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param pulumi.Input[_builtins.str] kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param pulumi.Input['_meta.v1.ListMetaArgs'] metadata: Standard list metadata
        """
        pulumi.set(__self__, "items", items)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'resource.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'DeviceClassList')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def items(self) -> pulumi.Input[Sequence[pulumi.Input['DeviceClassArgs']]]:
        """
        Items is the list of resource classes.
        """
        return pulumi.get(self, "items")

    @items.setter
    def items(self, value: pulumi.Input[Sequence[pulumi.Input['DeviceClassArgs']]]):
        pulumi.set(self, "items", value)

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @api_version.setter
    def api_version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "api_version", value)

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "kind", value)

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input['_meta.v1.ListMetaArgs']]:
        """
        Standard list metadata
        """
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input['_meta.v1.ListMetaArgs']]):
        pulumi.set(self, "metadata", value)


@pulumi.type_token("kubernetes:resource.k8s.io/v1beta1:DeviceClassList")
class DeviceClassList(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 items: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DeviceClassArgs', 'DeviceClassArgsDict']]]]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input[Union['_meta.v1.ListMetaArgs', '_meta.v1.ListMetaArgsDict']]] = None,
                 __props__=None):
        """
        DeviceClassList is a collection of classes.


        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param pulumi.Input[Sequence[pulumi.Input[Union['DeviceClassArgs', 'DeviceClassArgsDict']]]] items: Items is the list of resource classes.
        :param pulumi.Input[_builtins.str] kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param pulumi.Input[Union['_meta.v1.ListMetaArgs', '_meta.v1.ListMetaArgsDict']] metadata: Standard list metadata
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: DeviceClassListArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        DeviceClassList is a collection of classes.


        :param str resource_name: The name of the resource.
        :param DeviceClassListArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DeviceClassListArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 items: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DeviceClassArgs', 'DeviceClassArgsDict']]]]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input[Union['_meta.v1.ListMetaArgs', '_meta.v1.ListMetaArgsDict']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DeviceClassListArgs.__new__(DeviceClassListArgs)

            __props__.__dict__["api_version"] = 'resource.k8s.io/v1beta1'
            if items is None and not opts.urn:
                raise TypeError("Missing required property 'items'")
            __props__.__dict__["items"] = items
            __props__.__dict__["kind"] = 'DeviceClassList'
            __props__.__dict__["metadata"] = metadata
        super(DeviceClassList, __self__).__init__(
            'kubernetes:resource.k8s.io/v1beta1:DeviceClassList',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'DeviceClassList':
        """
        Get an existing DeviceClassList resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = DeviceClassListArgs.__new__(DeviceClassListArgs)

        __props__.__dict__["api_version"] = None
        __props__.__dict__["items"] = None
        __props__.__dict__["kind"] = None
        __props__.__dict__["metadata"] = None
        return DeviceClassList(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> pulumi.Output[_builtins.str]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @_builtins.property
    @pulumi.getter
    def items(self) -> pulumi.Output[Sequence['outputs.DeviceClass']]:
        """
        Items is the list of resource classes.
        """
        return pulumi.get(self, "items")

    @_builtins.property
    @pulumi.getter
    def kind(self) -> pulumi.Output[_builtins.str]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> pulumi.Output['_meta.v1.outputs.ListMeta']:
        """
        Standard list metadata
        """
        return pulumi.get(self, "metadata")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from ... import _utilities
from ... import meta as _meta

__all__ = [
    'CertificateSigningRequestConditionArgs',
    'CertificateSigningRequestConditionArgsDict',
    'CertificateSigningRequestSpecPatchArgs',
    'CertificateSigningRequestSpecPatchArgsDict',
    'CertificateSigningRequestSpecArgs',
    'CertificateSigningRequestSpecArgsDict',
    'CertificateSigningRequestStatusArgs',
    'CertificateSigningRequestStatusArgsDict',
    'CertificateSigningRequestArgs',
    'CertificateSigningRequestArgsDict',
    'ClusterTrustBundleSpecPatchArgs',
    'ClusterTrustBundleSpecPatchArgsDict',
    'ClusterTrustBundleSpecArgs',
    'ClusterTrustBundleSpecArgsDict',
    'ClusterTrustBundleArgs',
    'ClusterTrustBundleArgsDict',
    'PodCertificateRequestSpecPatchArgs',
    'PodCertificateRequestSpecPatchArgsDict',
    'PodCertificateRequestSpecArgs',
    'PodCertificateRequestSpecArgsDict',
    'PodCertificateRequestStatusArgs',
    'PodCertificateRequestStatusArgsDict',
    'PodCertificateRequestArgs',
    'PodCertificateRequestArgsDict',
]

class CertificateSigningRequestConditionArgsDict(TypedDict):
    type: pulumi.Input[_builtins.str]
    """
    request approval state, currently Approved or Denied.
    """
    last_transition_time: NotRequired[pulumi.Input[_builtins.str]]
    """
    lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.
    """
    last_update_time: NotRequired[pulumi.Input[_builtins.str]]
    """
    timestamp for the last update to this condition
    """
    message: NotRequired[pulumi.Input[_builtins.str]]
    """
    human readable message with details about the request state
    """
    reason: NotRequired[pulumi.Input[_builtins.str]]
    """
    brief reason for the request state
    """
    status: NotRequired[pulumi.Input[_builtins.str]]
    """
    Status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be "False" or "Unknown". Defaults to "True". If unset, should be treated as "True".
    """

@pulumi.input_type
class CertificateSigningRequestConditionArgs:
    def __init__(__self__, *,
                 type: pulumi.Input[_builtins.str],
                 last_transition_time: Optional[pulumi.Input[_builtins.str]] = None,
                 last_update_time: Optional[pulumi.Input[_builtins.str]] = None,
                 message: Optional[pulumi.Input[_builtins.str]] = None,
                 reason: Optional[pulumi.Input[_builtins.str]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] type: request approval state, currently Approved or Denied.
        :param pulumi.Input[_builtins.str] last_transition_time: lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.
        :param pulumi.Input[_builtins.str] last_update_time: timestamp for the last update to this condition
        :param pulumi.Input[_builtins.str] message: human readable message with details about the request state
        :param pulumi.Input[_builtins.str] reason: brief reason for the request state
        :param pulumi.Input[_builtins.str] status: Status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be "False" or "Unknown". Defaults to "True". If unset, should be treated as "True".
        """
        pulumi.set(__self__, "type", type)
        if last_transition_time is not None:
            pulumi.set(__self__, "last_transition_time", last_transition_time)
        if last_update_time is not None:
            pulumi.set(__self__, "last_update_time", last_update_time)
        if message is not None:
            pulumi.set(__self__, "message", message)
        if reason is not None:
            pulumi.set(__self__, "reason", reason)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Input[_builtins.str]:
        """
        request approval state, currently Approved or Denied.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter(name="lastTransitionTime")
    def last_transition_time(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.
        """
        return pulumi.get(self, "last_transition_time")

    @last_transition_time.setter
    def last_transition_time(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "last_transition_time", value)

    @_builtins.property
    @pulumi.getter(name="lastUpdateTime")
    def last_update_time(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        timestamp for the last update to this condition
        """
        return pulumi.get(self, "last_update_time")

    @last_update_time.setter
    def last_update_time(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "last_update_time", value)

    @_builtins.property
    @pulumi.getter
    def message(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        human readable message with details about the request state
        """
        return pulumi.get(self, "message")

    @message.setter
    def message(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "message", value)

    @_builtins.property
    @pulumi.getter
    def reason(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        brief reason for the request state
        """
        return pulumi.get(self, "reason")

    @reason.setter
    def reason(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "reason", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be "False" or "Unknown". Defaults to "True". If unset, should be treated as "True".
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)


class CertificateSigningRequestSpecPatchArgsDict(TypedDict):
    """
    This information is immutable after the request is created. Only the Request and Usages fields can be set on creation, other fields are derived by Kubernetes and cannot be modified by users.
    """
    extra: NotRequired[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]
    """
    Extra information about the requesting user. See user.Info interface for details.
    """
    groups: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
    """
    Group information about the requesting user. See user.Info interface for details.
    """
    request: NotRequired[pulumi.Input[_builtins.str]]
    """
    Base64-encoded PKCS#10 CSR data
    """
    signer_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
     1. If it's a kubelet client certificate, it is assigned
        "kubernetes.io/kube-apiserver-client-kubelet".
     2. If it's a kubelet serving certificate, it is assigned
        "kubernetes.io/kubelet-serving".
     3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
    Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
    """
    uid: NotRequired[pulumi.Input[_builtins.str]]
    """
    UID information about the requesting user. See user.Info interface for details.
    """
    usages: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
    """
    allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
         https://tools.ietf.org/html/rfc5280#section-4.2.1.12
    """
    username: NotRequired[pulumi.Input[_builtins.str]]
    """
    Information about the requesting user. See user.Info interface for details.
    """

@pulumi.input_type
class CertificateSigningRequestSpecPatchArgs:
    def __init__(__self__, *,
                 extra: Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 request: Optional[pulumi.Input[_builtins.str]] = None,
                 signer_name: Optional[pulumi.Input[_builtins.str]] = None,
                 uid: Optional[pulumi.Input[_builtins.str]] = None,
                 usages: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 username: Optional[pulumi.Input[_builtins.str]] = None):
        """
        This information is immutable after the request is created. Only the Request and Usages fields can be set on creation, other fields are derived by Kubernetes and cannot be modified by users.

        :param pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]] extra: Extra information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] groups: Group information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[_builtins.str] request: Base64-encoded PKCS#10 CSR data
        :param pulumi.Input[_builtins.str] signer_name: Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
                1. If it's a kubelet client certificate, it is assigned
                   "kubernetes.io/kube-apiserver-client-kubelet".
                2. If it's a kubelet serving certificate, it is assigned
                   "kubernetes.io/kubelet-serving".
                3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
               Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
        :param pulumi.Input[_builtins.str] uid: UID information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] usages: allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
                    https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        :param pulumi.Input[_builtins.str] username: Information about the requesting user. See user.Info interface for details.
        """
        if extra is not None:
            pulumi.set(__self__, "extra", extra)
        if groups is not None:
            pulumi.set(__self__, "groups", groups)
        if request is not None:
            pulumi.set(__self__, "request", request)
        if signer_name is not None:
            pulumi.set(__self__, "signer_name", signer_name)
        if uid is not None:
            pulumi.set(__self__, "uid", uid)
        if usages is not None:
            pulumi.set(__self__, "usages", usages)
        if username is not None:
            pulumi.set(__self__, "username", username)

    @_builtins.property
    @pulumi.getter
    def extra(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]:
        """
        Extra information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "extra")

    @extra.setter
    def extra(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]):
        pulumi.set(self, "extra", value)

    @_builtins.property
    @pulumi.getter
    def groups(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        Group information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "groups")

    @groups.setter
    def groups(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "groups", value)

    @_builtins.property
    @pulumi.getter
    def request(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Base64-encoded PKCS#10 CSR data
        """
        return pulumi.get(self, "request")

    @request.setter
    def request(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "request", value)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
         1. If it's a kubelet client certificate, it is assigned
            "kubernetes.io/kube-apiserver-client-kubelet".
         2. If it's a kubelet serving certificate, it is assigned
            "kubernetes.io/kubelet-serving".
         3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
        Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "signer_name", value)

    @_builtins.property
    @pulumi.getter
    def uid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        UID information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "uid")

    @uid.setter
    def uid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "uid", value)

    @_builtins.property
    @pulumi.getter
    def usages(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
             https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        """
        return pulumi.get(self, "usages")

    @usages.setter
    def usages(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "usages", value)

    @_builtins.property
    @pulumi.getter
    def username(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "username", value)


class CertificateSigningRequestSpecArgsDict(TypedDict):
    """
    This information is immutable after the request is created. Only the Request and Usages fields can be set on creation, other fields are derived by Kubernetes and cannot be modified by users.
    """
    request: pulumi.Input[_builtins.str]
    """
    Base64-encoded PKCS#10 CSR data
    """
    extra: NotRequired[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]
    """
    Extra information about the requesting user. See user.Info interface for details.
    """
    groups: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
    """
    Group information about the requesting user. See user.Info interface for details.
    """
    signer_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
     1. If it's a kubelet client certificate, it is assigned
        "kubernetes.io/kube-apiserver-client-kubelet".
     2. If it's a kubelet serving certificate, it is assigned
        "kubernetes.io/kubelet-serving".
     3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
    Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
    """
    uid: NotRequired[pulumi.Input[_builtins.str]]
    """
    UID information about the requesting user. See user.Info interface for details.
    """
    usages: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
    """
    allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
         https://tools.ietf.org/html/rfc5280#section-4.2.1.12
    """
    username: NotRequired[pulumi.Input[_builtins.str]]
    """
    Information about the requesting user. See user.Info interface for details.
    """

@pulumi.input_type
class CertificateSigningRequestSpecArgs:
    def __init__(__self__, *,
                 request: pulumi.Input[_builtins.str],
                 extra: Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signer_name: Optional[pulumi.Input[_builtins.str]] = None,
                 uid: Optional[pulumi.Input[_builtins.str]] = None,
                 usages: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 username: Optional[pulumi.Input[_builtins.str]] = None):
        """
        This information is immutable after the request is created. Only the Request and Usages fields can be set on creation, other fields are derived by Kubernetes and cannot be modified by users.

        :param pulumi.Input[_builtins.str] request: Base64-encoded PKCS#10 CSR data
        :param pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]] extra: Extra information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] groups: Group information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[_builtins.str] signer_name: Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
                1. If it's a kubelet client certificate, it is assigned
                   "kubernetes.io/kube-apiserver-client-kubelet".
                2. If it's a kubelet serving certificate, it is assigned
                   "kubernetes.io/kubelet-serving".
                3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
               Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
        :param pulumi.Input[_builtins.str] uid: UID information about the requesting user. See user.Info interface for details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] usages: allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
                    https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        :param pulumi.Input[_builtins.str] username: Information about the requesting user. See user.Info interface for details.
        """
        pulumi.set(__self__, "request", request)
        if extra is not None:
            pulumi.set(__self__, "extra", extra)
        if groups is not None:
            pulumi.set(__self__, "groups", groups)
        if signer_name is not None:
            pulumi.set(__self__, "signer_name", signer_name)
        if uid is not None:
            pulumi.set(__self__, "uid", uid)
        if usages is not None:
            pulumi.set(__self__, "usages", usages)
        if username is not None:
            pulumi.set(__self__, "username", username)

    @_builtins.property
    @pulumi.getter
    def request(self) -> pulumi.Input[_builtins.str]:
        """
        Base64-encoded PKCS#10 CSR data
        """
        return pulumi.get(self, "request")

    @request.setter
    def request(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "request", value)

    @_builtins.property
    @pulumi.getter
    def extra(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]:
        """
        Extra information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "extra")

    @extra.setter
    def extra(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]]]):
        pulumi.set(self, "extra", value)

    @_builtins.property
    @pulumi.getter
    def groups(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        Group information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "groups")

    @groups.setter
    def groups(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "groups", value)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Requested signer for the request. It is a qualified name in the form: `scope-hostname.io/name`. If empty, it will be defaulted:
         1. If it's a kubelet client certificate, it is assigned
            "kubernetes.io/kube-apiserver-client-kubelet".
         2. If it's a kubelet serving certificate, it is assigned
            "kubernetes.io/kubelet-serving".
         3. Otherwise, it is assigned "kubernetes.io/legacy-unknown".
        Distribution of trust for signers happens out of band. You can select on this field using `spec.signerName`.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "signer_name", value)

    @_builtins.property
    @pulumi.getter
    def uid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        UID information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "uid")

    @uid.setter
    def uid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "uid", value)

    @_builtins.property
    @pulumi.getter
    def usages(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3
             https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        """
        return pulumi.get(self, "usages")

    @usages.setter
    def usages(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "usages", value)

    @_builtins.property
    @pulumi.getter
    def username(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Information about the requesting user. See user.Info interface for details.
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "username", value)


class CertificateSigningRequestStatusArgsDict(TypedDict):
    certificate: NotRequired[pulumi.Input[_builtins.str]]
    """
    If request was approved, the controller will place the issued certificate here.
    """
    conditions: NotRequired[pulumi.Input[Sequence[pulumi.Input['CertificateSigningRequestConditionArgsDict']]]]
    """
    Conditions applied to the request, such as approval or denial.
    """

@pulumi.input_type
class CertificateSigningRequestStatusArgs:
    def __init__(__self__, *,
                 certificate: Optional[pulumi.Input[_builtins.str]] = None,
                 conditions: Optional[pulumi.Input[Sequence[pulumi.Input['CertificateSigningRequestConditionArgs']]]] = None):
        """
        :param pulumi.Input[_builtins.str] certificate: If request was approved, the controller will place the issued certificate here.
        :param pulumi.Input[Sequence[pulumi.Input['CertificateSigningRequestConditionArgs']]] conditions: Conditions applied to the request, such as approval or denial.
        """
        if certificate is not None:
            pulumi.set(__self__, "certificate", certificate)
        if conditions is not None:
            pulumi.set(__self__, "conditions", conditions)

    @_builtins.property
    @pulumi.getter
    def certificate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        If request was approved, the controller will place the issued certificate here.
        """
        return pulumi.get(self, "certificate")

    @certificate.setter
    def certificate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "certificate", value)

    @_builtins.property
    @pulumi.getter
    def conditions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['CertificateSigningRequestConditionArgs']]]]:
        """
        Conditions applied to the request, such as approval or denial.
        """
        return pulumi.get(self, "conditions")

    @conditions.setter
    def conditions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['CertificateSigningRequestConditionArgs']]]]):
        pulumi.set(self, "conditions", value)


class CertificateSigningRequestArgsDict(TypedDict):
    """
    Describes a certificate signing request
    """
    api_version: NotRequired[pulumi.Input[_builtins.str]]
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: NotRequired[pulumi.Input[_builtins.str]]
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: NotRequired[pulumi.Input['_meta.v1.ObjectMetaArgsDict']]
    spec: NotRequired[pulumi.Input['CertificateSigningRequestSpecArgsDict']]
    """
    The certificate request itself and any additional information.
    """
    status: NotRequired[pulumi.Input['CertificateSigningRequestStatusArgsDict']]
    """
    Derived information about the request.
    """

@pulumi.input_type
class CertificateSigningRequestArgs:
    def __init__(__self__, *,
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']] = None,
                 spec: Optional[pulumi.Input['CertificateSigningRequestSpecArgs']] = None,
                 status: Optional[pulumi.Input['CertificateSigningRequestStatusArgs']] = None):
        """
        Describes a certificate signing request

        :param pulumi.Input[_builtins.str] api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param pulumi.Input[_builtins.str] kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param pulumi.Input['CertificateSigningRequestSpecArgs'] spec: The certificate request itself and any additional information.
        :param pulumi.Input['CertificateSigningRequestStatusArgs'] status: Derived information about the request.
        """
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'certificates.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'CertificateSigningRequest')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if spec is not None:
            pulumi.set(__self__, "spec", spec)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @api_version.setter
    def api_version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "api_version", value)

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "kind", value)

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]:
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]):
        pulumi.set(self, "metadata", value)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> Optional[pulumi.Input['CertificateSigningRequestSpecArgs']]:
        """
        The certificate request itself and any additional information.
        """
        return pulumi.get(self, "spec")

    @spec.setter
    def spec(self, value: Optional[pulumi.Input['CertificateSigningRequestSpecArgs']]):
        pulumi.set(self, "spec", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input['CertificateSigningRequestStatusArgs']]:
        """
        Derived information about the request.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input['CertificateSigningRequestStatusArgs']]):
        pulumi.set(self, "status", value)


class ClusterTrustBundleSpecPatchArgsDict(TypedDict):
    """
    ClusterTrustBundleSpec contains the signer and trust anchors.
    """
    signer_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    signerName indicates the associated signer, if any.

    In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

    If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

    If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

    List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
    """
    trust_bundle: NotRequired[pulumi.Input[_builtins.str]]
    """
    trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

    The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

    Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
    """

@pulumi.input_type
class ClusterTrustBundleSpecPatchArgs:
    def __init__(__self__, *,
                 signer_name: Optional[pulumi.Input[_builtins.str]] = None,
                 trust_bundle: Optional[pulumi.Input[_builtins.str]] = None):
        """
        ClusterTrustBundleSpec contains the signer and trust anchors.

        :param pulumi.Input[_builtins.str] signer_name: signerName indicates the associated signer, if any.
               
               In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.
               
               If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.
               
               If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.
               
               List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
        :param pulumi.Input[_builtins.str] trust_bundle: trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.
               
               The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.
               
               Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
        """
        if signer_name is not None:
            pulumi.set(__self__, "signer_name", signer_name)
        if trust_bundle is not None:
            pulumi.set(__self__, "trust_bundle", trust_bundle)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        signerName indicates the associated signer, if any.

        In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

        If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

        If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

        List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "signer_name", value)

    @_builtins.property
    @pulumi.getter(name="trustBundle")
    def trust_bundle(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

        The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

        Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
        """
        return pulumi.get(self, "trust_bundle")

    @trust_bundle.setter
    def trust_bundle(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "trust_bundle", value)


class ClusterTrustBundleSpecArgsDict(TypedDict):
    """
    ClusterTrustBundleSpec contains the signer and trust anchors.
    """
    trust_bundle: pulumi.Input[_builtins.str]
    """
    trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

    The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

    Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
    """
    signer_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    signerName indicates the associated signer, if any.

    In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

    If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

    If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

    List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
    """

@pulumi.input_type
class ClusterTrustBundleSpecArgs:
    def __init__(__self__, *,
                 trust_bundle: pulumi.Input[_builtins.str],
                 signer_name: Optional[pulumi.Input[_builtins.str]] = None):
        """
        ClusterTrustBundleSpec contains the signer and trust anchors.

        :param pulumi.Input[_builtins.str] trust_bundle: trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.
               
               The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.
               
               Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
        :param pulumi.Input[_builtins.str] signer_name: signerName indicates the associated signer, if any.
               
               In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.
               
               If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.
               
               If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.
               
               List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
        """
        pulumi.set(__self__, "trust_bundle", trust_bundle)
        if signer_name is not None:
            pulumi.set(__self__, "signer_name", signer_name)

    @_builtins.property
    @pulumi.getter(name="trustBundle")
    def trust_bundle(self) -> pulumi.Input[_builtins.str]:
        """
        trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

        The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

        Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
        """
        return pulumi.get(self, "trust_bundle")

    @trust_bundle.setter
    def trust_bundle(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "trust_bundle", value)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        signerName indicates the associated signer, if any.

        In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

        If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

        If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

        List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "signer_name", value)


class ClusterTrustBundleArgsDict(TypedDict):
    """
    ClusterTrustBundle is a cluster-scoped container for X.509 trust anchors (root certificates).

    ClusterTrustBundle objects are considered to be readable by any authenticated user in the cluster, because they can be mounted by pods using the `clusterTrustBundle` projection.  All service accounts have read access to ClusterTrustBundles by default.  Users who only have namespace-level access to a cluster can read ClusterTrustBundles by impersonating a serviceaccount that they have access to.

    It can be optionally associated with a particular assigner, in which case it contains one valid set of trust anchors for that signer. Signers may have multiple associated ClusterTrustBundles; each is an independent set of trust anchors for that signer. Admission control is used to enforce that only users with permissions on the signer can create or modify the corresponding bundle.
    """
    spec: pulumi.Input['ClusterTrustBundleSpecArgsDict']
    """
    spec contains the signer (if any) and trust anchors.
    """
    api_version: NotRequired[pulumi.Input[_builtins.str]]
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: NotRequired[pulumi.Input[_builtins.str]]
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: NotRequired[pulumi.Input['_meta.v1.ObjectMetaArgsDict']]
    """
    metadata contains the object metadata.
    """

@pulumi.input_type
class ClusterTrustBundleArgs:
    def __init__(__self__, *,
                 spec: pulumi.Input['ClusterTrustBundleSpecArgs'],
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']] = None):
        """
        ClusterTrustBundle is a cluster-scoped container for X.509 trust anchors (root certificates).

        ClusterTrustBundle objects are considered to be readable by any authenticated user in the cluster, because they can be mounted by pods using the `clusterTrustBundle` projection.  All service accounts have read access to ClusterTrustBundles by default.  Users who only have namespace-level access to a cluster can read ClusterTrustBundles by impersonating a serviceaccount that they have access to.

        It can be optionally associated with a particular assigner, in which case it contains one valid set of trust anchors for that signer. Signers may have multiple associated ClusterTrustBundles; each is an independent set of trust anchors for that signer. Admission control is used to enforce that only users with permissions on the signer can create or modify the corresponding bundle.

        :param pulumi.Input['ClusterTrustBundleSpecArgs'] spec: spec contains the signer (if any) and trust anchors.
        :param pulumi.Input[_builtins.str] api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param pulumi.Input[_builtins.str] kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param pulumi.Input['_meta.v1.ObjectMetaArgs'] metadata: metadata contains the object metadata.
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'certificates.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'ClusterTrustBundle')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> pulumi.Input['ClusterTrustBundleSpecArgs']:
        """
        spec contains the signer (if any) and trust anchors.
        """
        return pulumi.get(self, "spec")

    @spec.setter
    def spec(self, value: pulumi.Input['ClusterTrustBundleSpecArgs']):
        pulumi.set(self, "spec", value)

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @api_version.setter
    def api_version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "api_version", value)

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "kind", value)

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]:
        """
        metadata contains the object metadata.
        """
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]):
        pulumi.set(self, "metadata", value)


class PodCertificateRequestSpecPatchArgsDict(TypedDict):
    """
    PodCertificateRequestSpec describes the certificate request.  All fields are immutable after creation.
    """
    max_expiration_seconds: NotRequired[pulumi.Input[_builtins.int]]
    """
    maxExpirationSeconds is the maximum lifetime permitted for the certificate.

    If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).

    The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
    """
    node_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    nodeName is the name of the node the pod is assigned to.
    """
    node_uid: NotRequired[pulumi.Input[_builtins.str]]
    """
    nodeUID is the UID of the node the pod is assigned to.
    """
    pkix_public_key: NotRequired[pulumi.Input[_builtins.str]]
    """
    The PKIX-serialized public key the signer will issue the certificate to.

    The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

    Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.

    Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
    """
    pod_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    podName is the name of the pod into which the certificate will be mounted.
    """
    pod_uid: NotRequired[pulumi.Input[_builtins.str]]
    """
    podUID is the UID of the pod into which the certificate will be mounted.
    """
    proof_of_possession: NotRequired[pulumi.Input[_builtins.str]]
    """
    A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.

    It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.

    kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.

    If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).

    If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)

    If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).

    Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
    """
    service_account_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    serviceAccountName is the name of the service account the pod is running as.
    """
    service_account_uid: NotRequired[pulumi.Input[_builtins.str]]
    """
    serviceAccountUID is the UID of the service account the pod is running as.
    """
    signer_name: NotRequired[pulumi.Input[_builtins.str]]
    """
    signerName indicates the requested signer.

    All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
    """
    stub_pkcs10_request: NotRequired[pulumi.Input[_builtins.str]]
    """
    A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.

    Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.

    The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

    Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
    """
    unverified_user_annotations: NotRequired[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]
    """
    unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.

    Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.

    Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
    """

@pulumi.input_type
class PodCertificateRequestSpecPatchArgs:
    def __init__(__self__, *,
                 max_expiration_seconds: Optional[pulumi.Input[_builtins.int]] = None,
                 node_name: Optional[pulumi.Input[_builtins.str]] = None,
                 node_uid: Optional[pulumi.Input[_builtins.str]] = None,
                 pkix_public_key: Optional[pulumi.Input[_builtins.str]] = None,
                 pod_name: Optional[pulumi.Input[_builtins.str]] = None,
                 pod_uid: Optional[pulumi.Input[_builtins.str]] = None,
                 proof_of_possession: Optional[pulumi.Input[_builtins.str]] = None,
                 service_account_name: Optional[pulumi.Input[_builtins.str]] = None,
                 service_account_uid: Optional[pulumi.Input[_builtins.str]] = None,
                 signer_name: Optional[pulumi.Input[_builtins.str]] = None,
                 stub_pkcs10_request: Optional[pulumi.Input[_builtins.str]] = None,
                 unverified_user_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None):
        """
        PodCertificateRequestSpec describes the certificate request.  All fields are immutable after creation.

        :param pulumi.Input[_builtins.int] max_expiration_seconds: maxExpirationSeconds is the maximum lifetime permitted for the certificate.
               
               If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).
               
               The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
        :param pulumi.Input[_builtins.str] node_name: nodeName is the name of the node the pod is assigned to.
        :param pulumi.Input[_builtins.str] node_uid: nodeUID is the UID of the node the pod is assigned to.
        :param pulumi.Input[_builtins.str] pkix_public_key: The PKIX-serialized public key the signer will issue the certificate to.
               
               The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.
               
               Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
               
               Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
        :param pulumi.Input[_builtins.str] pod_name: podName is the name of the pod into which the certificate will be mounted.
        :param pulumi.Input[_builtins.str] pod_uid: podUID is the UID of the pod into which the certificate will be mounted.
        :param pulumi.Input[_builtins.str] proof_of_possession: A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.
               
               It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.
               
               kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.
               
               If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).
               
               If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)
               
               If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).
               
               Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
        :param pulumi.Input[_builtins.str] service_account_name: serviceAccountName is the name of the service account the pod is running as.
        :param pulumi.Input[_builtins.str] service_account_uid: serviceAccountUID is the UID of the service account the pod is running as.
        :param pulumi.Input[_builtins.str] signer_name: signerName indicates the requested signer.
               
               All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
        :param pulumi.Input[_builtins.str] stub_pkcs10_request: A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.
               
               Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.
               
               The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.
               
               Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] unverified_user_annotations: unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.
               
               Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.
               
               Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
        """
        if max_expiration_seconds is not None:
            pulumi.set(__self__, "max_expiration_seconds", max_expiration_seconds)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if node_uid is not None:
            pulumi.set(__self__, "node_uid", node_uid)
        if pkix_public_key is not None:
            pulumi.set(__self__, "pkix_public_key", pkix_public_key)
        if pod_name is not None:
            pulumi.set(__self__, "pod_name", pod_name)
        if pod_uid is not None:
            pulumi.set(__self__, "pod_uid", pod_uid)
        if proof_of_possession is not None:
            pulumi.set(__self__, "proof_of_possession", proof_of_possession)
        if service_account_name is not None:
            pulumi.set(__self__, "service_account_name", service_account_name)
        if service_account_uid is not None:
            pulumi.set(__self__, "service_account_uid", service_account_uid)
        if signer_name is not None:
            pulumi.set(__self__, "signer_name", signer_name)
        if stub_pkcs10_request is not None:
            pulumi.set(__self__, "stub_pkcs10_request", stub_pkcs10_request)
        if unverified_user_annotations is not None:
            pulumi.set(__self__, "unverified_user_annotations", unverified_user_annotations)

    @_builtins.property
    @pulumi.getter(name="maxExpirationSeconds")
    def max_expiration_seconds(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        maxExpirationSeconds is the maximum lifetime permitted for the certificate.

        If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).

        The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
        """
        return pulumi.get(self, "max_expiration_seconds")

    @max_expiration_seconds.setter
    def max_expiration_seconds(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "max_expiration_seconds", value)

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        nodeName is the name of the node the pod is assigned to.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "node_name", value)

    @_builtins.property
    @pulumi.getter(name="nodeUID")
    def node_uid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        nodeUID is the UID of the node the pod is assigned to.
        """
        return pulumi.get(self, "node_uid")

    @node_uid.setter
    def node_uid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "node_uid", value)

    @_builtins.property
    @pulumi.getter(name="pkixPublicKey")
    def pkix_public_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The PKIX-serialized public key the signer will issue the certificate to.

        The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

        Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.

        Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
        """
        return pulumi.get(self, "pkix_public_key")

    @pkix_public_key.setter
    def pkix_public_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "pkix_public_key", value)

    @_builtins.property
    @pulumi.getter(name="podName")
    def pod_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        podName is the name of the pod into which the certificate will be mounted.
        """
        return pulumi.get(self, "pod_name")

    @pod_name.setter
    def pod_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "pod_name", value)

    @_builtins.property
    @pulumi.getter(name="podUID")
    def pod_uid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        podUID is the UID of the pod into which the certificate will be mounted.
        """
        return pulumi.get(self, "pod_uid")

    @pod_uid.setter
    def pod_uid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "pod_uid", value)

    @_builtins.property
    @pulumi.getter(name="proofOfPossession")
    def proof_of_possession(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.

        It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.

        kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.

        If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).

        If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)

        If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).

        Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
        """
        return pulumi.get(self, "proof_of_possession")

    @proof_of_possession.setter
    def proof_of_possession(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "proof_of_possession", value)

    @_builtins.property
    @pulumi.getter(name="serviceAccountName")
    def service_account_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        serviceAccountName is the name of the service account the pod is running as.
        """
        return pulumi.get(self, "service_account_name")

    @service_account_name.setter
    def service_account_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_account_name", value)

    @_builtins.property
    @pulumi.getter(name="serviceAccountUID")
    def service_account_uid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        serviceAccountUID is the UID of the service account the pod is running as.
        """
        return pulumi.get(self, "service_account_uid")

    @service_account_uid.setter
    def service_account_uid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_account_uid", value)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        signerName indicates the requested signer.

        All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "signer_name", value)

    @_builtins.property
    @pulumi.getter(name="stubPKCS10Request")
    def stub_pkcs10_request(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.

        Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.

        The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

        Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
        """
        return pulumi.get(self, "stub_pkcs10_request")

    @stub_pkcs10_request.setter
    def stub_pkcs10_request(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "stub_pkcs10_request", value)

    @_builtins.property
    @pulumi.getter(name="unverifiedUserAnnotations")
    def unverified_user_annotations(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]:
        """
        unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.

        Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.

        Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
        """
        return pulumi.get(self, "unverified_user_annotations")

    @unverified_user_annotations.setter
    def unverified_user_annotations(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "unverified_user_annotations", value)


class PodCertificateRequestSpecArgsDict(TypedDict):
    """
    PodCertificateRequestSpec describes the certificate request.  All fields are immutable after creation.
    """
    node_name: pulumi.Input[_builtins.str]
    """
    nodeName is the name of the node the pod is assigned to.
    """
    node_uid: pulumi.Input[_builtins.str]
    """
    nodeUID is the UID of the node the pod is assigned to.
    """
    pod_name: pulumi.Input[_builtins.str]
    """
    podName is the name of the pod into which the certificate will be mounted.
    """
    pod_uid: pulumi.Input[_builtins.str]
    """
    podUID is the UID of the pod into which the certificate will be mounted.
    """
    service_account_name: pulumi.Input[_builtins.str]
    """
    serviceAccountName is the name of the service account the pod is running as.
    """
    service_account_uid: pulumi.Input[_builtins.str]
    """
    serviceAccountUID is the UID of the service account the pod is running as.
    """
    signer_name: pulumi.Input[_builtins.str]
    """
    signerName indicates the requested signer.

    All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
    """
    stub_pkcs10_request: pulumi.Input[_builtins.str]
    """
    A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.

    Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.

    The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

    Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
    """
    max_expiration_seconds: NotRequired[pulumi.Input[_builtins.int]]
    """
    maxExpirationSeconds is the maximum lifetime permitted for the certificate.

    If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).

    The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
    """
    pkix_public_key: NotRequired[pulumi.Input[_builtins.str]]
    """
    The PKIX-serialized public key the signer will issue the certificate to.

    The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

    Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.

    Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
    """
    proof_of_possession: NotRequired[pulumi.Input[_builtins.str]]
    """
    A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.

    It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.

    kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.

    If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).

    If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)

    If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).

    Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
    """
    unverified_user_annotations: NotRequired[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]
    """
    unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.

    Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.

    Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
    """

@pulumi.input_type
class PodCertificateRequestSpecArgs:
    def __init__(__self__, *,
                 node_name: pulumi.Input[_builtins.str],
                 node_uid: pulumi.Input[_builtins.str],
                 pod_name: pulumi.Input[_builtins.str],
                 pod_uid: pulumi.Input[_builtins.str],
                 service_account_name: pulumi.Input[_builtins.str],
                 service_account_uid: pulumi.Input[_builtins.str],
                 signer_name: pulumi.Input[_builtins.str],
                 stub_pkcs10_request: pulumi.Input[_builtins.str],
                 max_expiration_seconds: Optional[pulumi.Input[_builtins.int]] = None,
                 pkix_public_key: Optional[pulumi.Input[_builtins.str]] = None,
                 proof_of_possession: Optional[pulumi.Input[_builtins.str]] = None,
                 unverified_user_annotations: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None):
        """
        PodCertificateRequestSpec describes the certificate request.  All fields are immutable after creation.

        :param pulumi.Input[_builtins.str] node_name: nodeName is the name of the node the pod is assigned to.
        :param pulumi.Input[_builtins.str] node_uid: nodeUID is the UID of the node the pod is assigned to.
        :param pulumi.Input[_builtins.str] pod_name: podName is the name of the pod into which the certificate will be mounted.
        :param pulumi.Input[_builtins.str] pod_uid: podUID is the UID of the pod into which the certificate will be mounted.
        :param pulumi.Input[_builtins.str] service_account_name: serviceAccountName is the name of the service account the pod is running as.
        :param pulumi.Input[_builtins.str] service_account_uid: serviceAccountUID is the UID of the service account the pod is running as.
        :param pulumi.Input[_builtins.str] signer_name: signerName indicates the requested signer.
               
               All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
        :param pulumi.Input[_builtins.str] stub_pkcs10_request: A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.
               
               Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.
               
               The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.
               
               Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
        :param pulumi.Input[_builtins.int] max_expiration_seconds: maxExpirationSeconds is the maximum lifetime permitted for the certificate.
               
               If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).
               
               The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
        :param pulumi.Input[_builtins.str] pkix_public_key: The PKIX-serialized public key the signer will issue the certificate to.
               
               The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.
               
               Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
               
               Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
        :param pulumi.Input[_builtins.str] proof_of_possession: A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.
               
               It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.
               
               kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.
               
               If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).
               
               If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)
               
               If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).
               
               Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] unverified_user_annotations: unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.
               
               Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.
               
               Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
        """
        pulumi.set(__self__, "node_name", node_name)
        pulumi.set(__self__, "node_uid", node_uid)
        pulumi.set(__self__, "pod_name", pod_name)
        pulumi.set(__self__, "pod_uid", pod_uid)
        pulumi.set(__self__, "service_account_name", service_account_name)
        pulumi.set(__self__, "service_account_uid", service_account_uid)
        pulumi.set(__self__, "signer_name", signer_name)
        pulumi.set(__self__, "stub_pkcs10_request", stub_pkcs10_request)
        if max_expiration_seconds is not None:
            pulumi.set(__self__, "max_expiration_seconds", max_expiration_seconds)
        if pkix_public_key is not None:
            pulumi.set(__self__, "pkix_public_key", pkix_public_key)
        if proof_of_possession is not None:
            pulumi.set(__self__, "proof_of_possession", proof_of_possession)
        if unverified_user_annotations is not None:
            pulumi.set(__self__, "unverified_user_annotations", unverified_user_annotations)

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[_builtins.str]:
        """
        nodeName is the name of the node the pod is assigned to.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "node_name", value)

    @_builtins.property
    @pulumi.getter(name="nodeUID")
    def node_uid(self) -> pulumi.Input[_builtins.str]:
        """
        nodeUID is the UID of the node the pod is assigned to.
        """
        return pulumi.get(self, "node_uid")

    @node_uid.setter
    def node_uid(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "node_uid", value)

    @_builtins.property
    @pulumi.getter(name="podName")
    def pod_name(self) -> pulumi.Input[_builtins.str]:
        """
        podName is the name of the pod into which the certificate will be mounted.
        """
        return pulumi.get(self, "pod_name")

    @pod_name.setter
    def pod_name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "pod_name", value)

    @_builtins.property
    @pulumi.getter(name="podUID")
    def pod_uid(self) -> pulumi.Input[_builtins.str]:
        """
        podUID is the UID of the pod into which the certificate will be mounted.
        """
        return pulumi.get(self, "pod_uid")

    @pod_uid.setter
    def pod_uid(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "pod_uid", value)

    @_builtins.property
    @pulumi.getter(name="serviceAccountName")
    def service_account_name(self) -> pulumi.Input[_builtins.str]:
        """
        serviceAccountName is the name of the service account the pod is running as.
        """
        return pulumi.get(self, "service_account_name")

    @service_account_name.setter
    def service_account_name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "service_account_name", value)

    @_builtins.property
    @pulumi.getter(name="serviceAccountUID")
    def service_account_uid(self) -> pulumi.Input[_builtins.str]:
        """
        serviceAccountUID is the UID of the service account the pod is running as.
        """
        return pulumi.get(self, "service_account_uid")

    @service_account_uid.setter
    def service_account_uid(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "service_account_uid", value)

    @_builtins.property
    @pulumi.getter(name="signerName")
    def signer_name(self) -> pulumi.Input[_builtins.str]:
        """
        signerName indicates the requested signer.

        All signer names beginning with `kubernetes.io` are reserved for use by the Kubernetes project.  There is currently one well-known signer documented by the Kubernetes project, `kubernetes.io/kube-apiserver-client-pod`, which will issue client certificates understood by kube-apiserver.  It is currently unimplemented.
        """
        return pulumi.get(self, "signer_name")

    @signer_name.setter
    def signer_name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "signer_name", value)

    @_builtins.property
    @pulumi.getter(name="stubPKCS10Request")
    def stub_pkcs10_request(self) -> pulumi.Input[_builtins.str]:
        """
        A PKCS#10 certificate signing request (DER-serialized) generated by Kubelet using the subject private key.

        Most signer implementations will ignore the contents of the CSR except to extract the subject public key. The API server automatically verifies the CSR signature during admission, so the signer does not need to repeat the verification.  CSRs generated by kubelet are completely empty.

        The subject public key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

        Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.
        """
        return pulumi.get(self, "stub_pkcs10_request")

    @stub_pkcs10_request.setter
    def stub_pkcs10_request(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "stub_pkcs10_request", value)

    @_builtins.property
    @pulumi.getter(name="maxExpirationSeconds")
    def max_expiration_seconds(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        maxExpirationSeconds is the maximum lifetime permitted for the certificate.

        If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).

        The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.
        """
        return pulumi.get(self, "max_expiration_seconds")

    @max_expiration_seconds.setter
    def max_expiration_seconds(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "max_expiration_seconds", value)

    @_builtins.property
    @pulumi.getter(name="pkixPublicKey")
    def pkix_public_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The PKIX-serialized public key the signer will issue the certificate to.

        The key must be one of RSA3072, RSA4096, ECDSAP256, ECDSAP384, ECDSAP521, or ED25519. Note that this list may be expanded in the future.

        Signer implementations do not need to support all key types supported by kube-apiserver and kubelet.  If a signer does not support the key type used for a given PodCertificateRequest, it must deny the request by setting a status.conditions entry with a type of "Denied" and a reason of "UnsupportedKeyType". It may also suggest a key type that it does support in the message field.

        Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.  Signer implementations should extract the public key from the StubPKCS10Request field.
        """
        return pulumi.get(self, "pkix_public_key")

    @pkix_public_key.setter
    def pkix_public_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "pkix_public_key", value)

    @_builtins.property
    @pulumi.getter(name="proofOfPossession")
    def proof_of_possession(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A proof that the requesting kubelet holds the private key corresponding to pkixPublicKey.

        It is contructed by signing the ASCII bytes of the pod's UID using `pkixPublicKey`.

        kube-apiserver validates the proof of possession during creation of the PodCertificateRequest.

        If the key is an RSA key, then the signature is over the ASCII bytes of the pod UID, using RSASSA-PSS from RFC 8017 (as implemented by the golang function crypto/rsa.SignPSS with nil options).

        If the key is an ECDSA key, then the signature is as described by [SEC 1, Version 2.0](https://www.secg.org/sec1-v2.pdf) (as implemented by the golang library function crypto/ecdsa.SignASN1)

        If the key is an ED25519 key, the the signature is as described by the [ED25519 Specification](https://ed25519.cr.yp.to/) (as implemented by the golang library crypto/ed25519.Sign).

        Deprecated: This field is replaced by StubPKCS10Request. If StubPKCS10Request is set, this field must be empty.
        """
        return pulumi.get(self, "proof_of_possession")

    @proof_of_possession.setter
    def proof_of_possession(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "proof_of_possession", value)

    @_builtins.property
    @pulumi.getter(name="unverifiedUserAnnotations")
    def unverified_user_annotations(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]:
        """
        unverifiedUserAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.

        Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.

        Signers should document the keys and values they support.  Signers should deny requests that contain keys they do not recognize.
        """
        return pulumi.get(self, "unverified_user_annotations")

    @unverified_user_annotations.setter
    def unverified_user_annotations(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "unverified_user_annotations", value)


class PodCertificateRequestStatusArgsDict(TypedDict):
    """
    PodCertificateRequestStatus describes the status of the request, and holds the certificate data if the request is issued.
    """
    begin_refresh_at: NotRequired[pulumi.Input[_builtins.str]]
    """
    beginRefreshAt is the time at which the kubelet should begin trying to refresh the certificate.  This field is set via the /status subresource, and must be set at the same time as certificateChain.  Once populated, this field is immutable.

    This field is only a hint.  Kubelet may start refreshing before or after this time if necessary.
    """
    certificate_chain: NotRequired[pulumi.Input[_builtins.str]]
    """
    certificateChain is populated with an issued certificate by the signer. This field is set via the /status subresource. Once populated, this field is immutable.

    If the certificate signing request is denied, a condition of type "Denied" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type "Failed" is added and this field remains empty.

    Validation requirements:
     1. certificateChain must consist of one or more PEM-formatted certificates.
     2. Each entry must be a valid PEM-wrapped, DER-encoded ASN.1 Certificate as
        described in section 4 of RFC5280.

    If more than one block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.  When projecting the chain into a pod volume, kubelet will drop any data in-between the PEM blocks, as well as any PEM block headers.
    """
    conditions: NotRequired[pulumi.Input[Sequence[pulumi.Input['_meta.v1.ConditionArgsDict']]]]
    """
    conditions applied to the request.

    The types "Issued", "Denied", and "Failed" have special handling.  At most one of these conditions may be present, and they must have status "True".

    If the request is denied with `Reason=UnsupportedKeyType`, the signer may suggest a key type that will work in the message field.
    """
    not_after: NotRequired[pulumi.Input[_builtins.str]]
    """
    notAfter is the time at which the certificate expires.  The value must be the same as the notAfter value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable.  The signer must set this field at the same time it sets certificateChain.
    """
    not_before: NotRequired[pulumi.Input[_builtins.str]]
    """
    notBefore is the time at which the certificate becomes valid.  The value must be the same as the notBefore value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable. The signer must set this field at the same time it sets certificateChain.
    """

@pulumi.input_type
class PodCertificateRequestStatusArgs:
    def __init__(__self__, *,
                 begin_refresh_at: Optional[pulumi.Input[_builtins.str]] = None,
                 certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
                 conditions: Optional[pulumi.Input[Sequence[pulumi.Input['_meta.v1.ConditionArgs']]]] = None,
                 not_after: Optional[pulumi.Input[_builtins.str]] = None,
                 not_before: Optional[pulumi.Input[_builtins.str]] = None):
        """
        PodCertificateRequestStatus describes the status of the request, and holds the certificate data if the request is issued.

        :param pulumi.Input[_builtins.str] begin_refresh_at: beginRefreshAt is the time at which the kubelet should begin trying to refresh the certificate.  This field is set via the /status subresource, and must be set at the same time as certificateChain.  Once populated, this field is immutable.
               
               This field is only a hint.  Kubelet may start refreshing before or after this time if necessary.
        :param pulumi.Input[_builtins.str] certificate_chain: certificateChain is populated with an issued certificate by the signer. This field is set via the /status subresource. Once populated, this field is immutable.
               
               If the certificate signing request is denied, a condition of type "Denied" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type "Failed" is added and this field remains empty.
               
               Validation requirements:
                1. certificateChain must consist of one or more PEM-formatted certificates.
                2. Each entry must be a valid PEM-wrapped, DER-encoded ASN.1 Certificate as
                   described in section 4 of RFC5280.
               
               If more than one block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.  When projecting the chain into a pod volume, kubelet will drop any data in-between the PEM blocks, as well as any PEM block headers.
        :param pulumi.Input[Sequence[pulumi.Input['_meta.v1.ConditionArgs']]] conditions: conditions applied to the request.
               
               The types "Issued", "Denied", and "Failed" have special handling.  At most one of these conditions may be present, and they must have status "True".
               
               If the request is denied with `Reason=UnsupportedKeyType`, the signer may suggest a key type that will work in the message field.
        :param pulumi.Input[_builtins.str] not_after: notAfter is the time at which the certificate expires.  The value must be the same as the notAfter value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable.  The signer must set this field at the same time it sets certificateChain.
        :param pulumi.Input[_builtins.str] not_before: notBefore is the time at which the certificate becomes valid.  The value must be the same as the notBefore value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable. The signer must set this field at the same time it sets certificateChain.
        """
        if begin_refresh_at is not None:
            pulumi.set(__self__, "begin_refresh_at", begin_refresh_at)
        if certificate_chain is not None:
            pulumi.set(__self__, "certificate_chain", certificate_chain)
        if conditions is not None:
            pulumi.set(__self__, "conditions", conditions)
        if not_after is not None:
            pulumi.set(__self__, "not_after", not_after)
        if not_before is not None:
            pulumi.set(__self__, "not_before", not_before)

    @_builtins.property
    @pulumi.getter(name="beginRefreshAt")
    def begin_refresh_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        beginRefreshAt is the time at which the kubelet should begin trying to refresh the certificate.  This field is set via the /status subresource, and must be set at the same time as certificateChain.  Once populated, this field is immutable.

        This field is only a hint.  Kubelet may start refreshing before or after this time if necessary.
        """
        return pulumi.get(self, "begin_refresh_at")

    @begin_refresh_at.setter
    def begin_refresh_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "begin_refresh_at", value)

    @_builtins.property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        certificateChain is populated with an issued certificate by the signer. This field is set via the /status subresource. Once populated, this field is immutable.

        If the certificate signing request is denied, a condition of type "Denied" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type "Failed" is added and this field remains empty.

        Validation requirements:
         1. certificateChain must consist of one or more PEM-formatted certificates.
         2. Each entry must be a valid PEM-wrapped, DER-encoded ASN.1 Certificate as
            described in section 4 of RFC5280.

        If more than one block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.  When projecting the chain into a pod volume, kubelet will drop any data in-between the PEM blocks, as well as any PEM block headers.
        """
        return pulumi.get(self, "certificate_chain")

    @certificate_chain.setter
    def certificate_chain(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "certificate_chain", value)

    @_builtins.property
    @pulumi.getter
    def conditions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['_meta.v1.ConditionArgs']]]]:
        """
        conditions applied to the request.

        The types "Issued", "Denied", and "Failed" have special handling.  At most one of these conditions may be present, and they must have status "True".

        If the request is denied with `Reason=UnsupportedKeyType`, the signer may suggest a key type that will work in the message field.
        """
        return pulumi.get(self, "conditions")

    @conditions.setter
    def conditions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['_meta.v1.ConditionArgs']]]]):
        pulumi.set(self, "conditions", value)

    @_builtins.property
    @pulumi.getter(name="notAfter")
    def not_after(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        notAfter is the time at which the certificate expires.  The value must be the same as the notAfter value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable.  The signer must set this field at the same time it sets certificateChain.
        """
        return pulumi.get(self, "not_after")

    @not_after.setter
    def not_after(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "not_after", value)

    @_builtins.property
    @pulumi.getter(name="notBefore")
    def not_before(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        notBefore is the time at which the certificate becomes valid.  The value must be the same as the notBefore value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable. The signer must set this field at the same time it sets certificateChain.
        """
        return pulumi.get(self, "not_before")

    @not_before.setter
    def not_before(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "not_before", value)


class PodCertificateRequestArgsDict(TypedDict):
    """
    PodCertificateRequest encodes a pod requesting a certificate from a given signer.

    Kubelets use this API to implement podCertificate projected volumes
    """
    spec: pulumi.Input['PodCertificateRequestSpecArgsDict']
    """
    spec contains the details about the certificate being requested.
    """
    api_version: NotRequired[pulumi.Input[_builtins.str]]
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: NotRequired[pulumi.Input[_builtins.str]]
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: NotRequired[pulumi.Input['_meta.v1.ObjectMetaArgsDict']]
    """
    metadata contains the object metadata.
    """
    status: NotRequired[pulumi.Input['PodCertificateRequestStatusArgsDict']]
    """
    status contains the issued certificate, and a standard set of conditions.
    """

@pulumi.input_type
class PodCertificateRequestArgs:
    def __init__(__self__, *,
                 spec: pulumi.Input['PodCertificateRequestSpecArgs'],
                 api_version: Optional[pulumi.Input[_builtins.str]] = None,
                 kind: Optional[pulumi.Input[_builtins.str]] = None,
                 metadata: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']] = None,
                 status: Optional[pulumi.Input['PodCertificateRequestStatusArgs']] = None):
        """
        PodCertificateRequest encodes a pod requesting a certificate from a given signer.

        Kubelets use this API to implement podCertificate projected volumes

        :param pulumi.Input['PodCertificateRequestSpecArgs'] spec: spec contains the details about the certificate being requested.
        :param pulumi.Input[_builtins.str] api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param pulumi.Input[_builtins.str] kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param pulumi.Input['_meta.v1.ObjectMetaArgs'] metadata: metadata contains the object metadata.
        :param pulumi.Input['PodCertificateRequestStatusArgs'] status: status contains the issued certificate, and a standard set of conditions.
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'certificates.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'PodCertificateRequest')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> pulumi.Input['PodCertificateRequestSpecArgs']:
        """
        spec contains the details about the certificate being requested.
        """
        return pulumi.get(self, "spec")

    @spec.setter
    def spec(self, value: pulumi.Input['PodCertificateRequestSpecArgs']):
        pulumi.set(self, "spec", value)

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @api_version.setter
    def api_version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "api_version", value)

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "kind", value)

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]:
        """
        metadata contains the object metadata.
        """
        return pulumi.get(self, "metadata")

    @metadata.setter
    def metadata(self, value: Optional[pulumi.Input['_meta.v1.ObjectMetaArgs']]):
        pulumi.set(self, "metadata", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input['PodCertificateRequestStatusArgs']]:
        """
        status contains the issued certificate, and a standard set of conditions.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input['PodCertificateRequestStatusArgs']]):
        pulumi.set(self, "status", value)


